local addonInfo, InternalInterface = ...
local addonID = addonInfo.identifier

local LOAD_DELAY = 1
local ITEM_FRAME_SIZE = 50
local ITEM_FRAME_MARGIN = 1.5
local ITEM_FRAME_PADDING = 4
local RIBBON_TOPBORDER_SIZE = 64
local RIBBON_BOTTOMBORDER_SIZE = 32
local RIBBON_SIDE_SIZE = 60

local CTooltip = Command.Tooltip
local CAScan = Command.Auction.Scan
local IInteraction = Inspect.Interaction
local IIDetail = Inspect.Item.Detail
local IIList = Inspect.Item.List
local ITFrame = Inspect.Time.Frame
local MAbs = math.abs
local MCeil = math.ceil
local MFloor = math.floor
local MMax = math.max
local MMin = math.min
local SFormat = string.format
local UISInventory = Utility.Item.Slot.Inventory
local TInsert = table.insert
local TSort = table.sort
local GetAuctions = LibPGC.GetActiveAuctionData
local PostItem = LibPGC.PostItem
local CancelAll = LibPGC.CancelAll
local GetPostingQueue = LibPGC.GetPostingQueue
local GetPostingQueuePaused = LibPGC.GetPostingQueuePaused
local SetPostingQueuePaused = LibPGC.SetPostingQueuePaused
local GetPostingQueueStatus = LibPGC.GetPostingQueueStatus
local L = Library.Translate

local dialog = false

local function InitializeSavedVars()
	APGadgetHiddenItems = APGadgetHiddenItems or {}
	APGadgetPostingConfigs = APGadgetPostingConfigs or {}
end

local function GetRarityColor(rarity)
	if     rarity == "sellable"     then return 0.34375, 0.34375, 0.34375, 1
	elseif rarity == "uncommon"     then return 0,       0.797,   0,       1
	elseif rarity == "rare"         then return 0.148,   0.496,   0.977,   1
	elseif rarity == "epic"         then return 0.676,   0.281,   0.98,    1
	elseif rarity == "relic"        then return 1,       0.5,     0,       1
	elseif rarity == "quest"        then return 1,       1,       0,       1
	elseif rarity == "transcendant" then return 1,       0.7,     0.9,     1
	else                                 return 0.98,    0.98,    0.98,    1
	end
end

local function CreateItemFrame(name, parent)
	local itemFrame = UI.CreateFrame("Frame", name, parent)
	local itemTexture = UI.CreateFrame("Texture", name .. ".ItemTexture", itemFrame)
	
	itemFrame.lastItemType = nil

	itemFrame:SetVisible(false)
	
	itemTexture:SetPoint("TOPLEFT", itemFrame, "TOPLEFT", ITEM_FRAME_MARGIN, ITEM_FRAME_MARGIN)
	itemTexture:SetPoint("BOTTOMRIGHT", itemFrame, "BOTTOMRIGHT", -ITEM_FRAME_MARGIN, -ITEM_FRAME_MARGIN)
	
	function itemFrame:SetItem(itemType, icon, rarity)
		itemFrame.lastItemType = itemType
		if itemType then
			itemFrame:SetVisible(true)
			itemFrame:SetBackgroundColor(GetRarityColor(rarity))
			itemTexture:SetTexture("Rift", icon)
			itemTexture:SetAlpha(APGadgetHiddenItems[itemType] and 0.5 or 1)
		else
			itemFrame:SetVisible(false)
		end
	end
	
	function itemTexture.Event:MouseIn() CTooltip(itemFrame.lastItemType) end
	function itemTexture.Event:MouseOut() CTooltip(nil) end
	function itemTexture.Event:LeftClick() itemFrame.Event.LeftClick(itemFrame) end
	function itemTexture.Event:RightClick() itemFrame.Event.RightClick(itemFrame) end
	
	return itemFrame
end

local function CreateGadget(configuration)
	configuration = configuration or {}
	local ITEM_FRAME_EXTRA = configuration.extraItems and MFloor((tonumber(configuration.extraItems) - 1) / 2) or 3
	local auctionOpacity = tonumber(configuration.auctionOpacity or "50") / 100
	local defaultPrice = configuration.defaultPrice or "average"
	local defaultDuration = tonumber(configuration.defaultDuration or "12") / 12
	if defaultDuration > 3 then defaultDuration = 3 end
    
	local itemRibbon = UI.CreateFrame("Frame", addonID .. ".UI.Main", WT.Context)
	
	local ribbonHeader = UI.CreateFrame("Texture", addonID .. ".UI.RibbonHeader", itemRibbon)
	local itemName = UI.CreateFrame("Text", addonID .. ".UI.ItemName", itemRibbon)
	local hideButton = UI.CreateFrame("Texture", addonID .. ".UI.HideButton", itemRibbon)
	local scanButton = UI.CreateFrame("Texture", addonID .. ".UI.ScanButton", itemRibbon)
	local foldButton = UI.CreateFrame("Texture", addonID .. ".UI.FoldButton", itemRibbon)
	local playButton = UI.CreateFrame("Texture", addonID .. ".UI.PlayButton", itemRibbon)
	local clearButton = UI.CreateFrame("Texture", addonID .. ".UI.ClearButton", itemRibbon)
	local queuePanel = UI.CreateFrame("BPanel", addonID .. ".UI.QueuePanel", itemRibbon)
	local queueText = UI.CreateFrame("Text", queuePanel:GetName() .. ".Text", queuePanel:GetContent())
	
	local auctionFrame = UI.CreateFrame("Frame", addonID .. ".UI.Auction", WT.Context)
	local auctionBackground = UI.CreateFrame("Texture", addonID .. ".UI.Auction.Background", auctionFrame)
	
	local averageText = UI.CreateFrame("Text", auctionFrame:GetName() .. ".AverageText", auctionFrame)
	local lowestText = UI.CreateFrame("Text", auctionFrame:GetName() .. ".LowestText", auctionFrame)
	local lastText = UI.CreateFrame("Text", auctionFrame:GetName() .. ".LastText", auctionFrame)
	local stackSizeText = UI.CreateFrame("Text", auctionFrame:GetName() .. ".StackSizeText", auctionFrame)
	local stackText = UI.CreateFrame("Text", auctionFrame:GetName() .. ".StackText", auctionFrame)
	local priceText = UI.CreateFrame("Text", auctionFrame:GetName() .. ".PriceText", auctionFrame)
	local durationText = UI.CreateFrame("Text", auctionFrame:GetName() .. ".DurationText", auctionFrame)
	local averageMoney = UI.CreateFrame("BMoneyDisplay", auctionFrame:GetName() .. ".DurationText", auctionFrame)
	local lowestMoney = UI.CreateFrame("BMoneyDisplay", auctionFrame:GetName() .. ".DurationText", auctionFrame)
	local lastMoney = UI.CreateFrame("BMoneyDisplay", auctionFrame:GetName() .. ".DurationText", auctionFrame)
	local stackSizeSlider = UI.CreateFrame("BSlider", auctionFrame:GetName() .. ".StackSizeSlider", auctionFrame)
	local stackSlider = UI.CreateFrame("BSlider", auctionFrame:GetName() .. ".StackSlider", auctionFrame)
	local priceSelector = UI.CreateFrame("BMoneySelector", auctionFrame:GetName() .. ".PriceSelector", auctionFrame)
	local durationSlider = UI.CreateFrame("RiftSlider", auctionFrame:GetName() .. ".DurationSlider", auctionFrame)
	local hoursText = UI.CreateFrame("Text", auctionFrame:GetName() .. ".HoursText", auctionFrame)
	local refreshButton = UI.CreateFrame("RiftButton", auctionFrame:GetName() .. ".RefreshButton", auctionFrame)
	local postButton = UI.CreateFrame("RiftButton", auctionFrame:GetName() .. ".PostButton", auctionFrame)

	local itemFrames = {}
	for index = 1, ITEM_FRAME_EXTRA * 2 + 1 do
		local offset = (index - ITEM_FRAME_EXTRA - 1) * (ITEM_FRAME_SIZE + ITEM_FRAME_PADDING)
		if offset ~= 0 then
			offset = offset + (offset / MAbs(offset)) * RIBBON_BOTTOMBORDER_SIZE / 2
		end
		
		itemFrames[index] = CreateItemFrame(addonID .. ".UI.ItemFrames." .. index, itemRibbon)
		itemFrames[index]:SetPoint("TOPCENTER", itemRibbon, "TOPCENTER", offset, RIBBON_TOPBORDER_SIZE)
		if index == ITEM_FRAME_EXTRA + 1 then
			itemFrames[index]:SetWidth(ITEM_FRAME_SIZE + RIBBON_BOTTOMBORDER_SIZE / 2)
			itemFrames[index]:SetHeight(ITEM_FRAME_SIZE + RIBBON_BOTTOMBORDER_SIZE / 2)
		else
			itemFrames[index]:SetWidth(ITEM_FRAME_SIZE)
			itemFrames[index]:SetHeight(ITEM_FRAME_SIZE)
		end
	end

	itemRibbon:SetWidth(RIBBON_SIDE_SIZE * 2 + ITEM_FRAME_EXTRA * 2 * (ITEM_FRAME_SIZE + ITEM_FRAME_PADDING) + ITEM_FRAME_SIZE + RIBBON_BOTTOMBORDER_SIZE * 2)
	itemRibbon:SetHeight(ITEM_FRAME_SIZE + RIBBON_TOPBORDER_SIZE + RIBBON_BOTTOMBORDER_SIZE)
	
	ribbonHeader:SetPoint("BOTTOMCENTER", itemRibbon, "TOPCENTER", 0, RIBBON_TOPBORDER_SIZE)
	ribbonHeader:SetWidth(itemRibbon:GetWidth())
	ribbonHeader:SetTexture(addonID, "Textures/Header.png")
	ribbonHeader:SetLayer(-9999)
	
	itemName:SetPoint("CENTER", ribbonHeader, "CENTER", 0, 0)
	itemName:SetFontSize(18)
	itemName:SetText("")
	itemName:SetFontColor(0.1, 0, 0, 1)
	
	hideButton:SetPoint("TOPRIGHT", itemFrames[1], "TOPLEFT", -10, 0)
	hideButton:SetTexture(addonID, "Textures/HideIcon.png")
	hideButton:SetWidth(24)
	hideButton:SetHeight(24)

	foldButton:SetPoint("BOTTOMRIGHT", itemFrames[1], "BOTTOMLEFT", -10, 0)
	foldButton:SetTexture(addonID, "Textures/ShowAuctionFrame.png")
	foldButton:SetWidth(24)
	foldButton:SetHeight(24)

	scanButton:SetPoint("BOTTOMRIGHT", foldButton, "BOTTOMLEFT", -10, 0)
	scanButton:SetTexture(addonID, "Textures/RefreshMiniDisabled.png")
	scanButton:SetWidth(24)
	scanButton:SetHeight(24)
	
	playButton:SetPoint("BOTTOMLEFT", itemFrames[#itemFrames], "BOTTOMRIGHT", 10, 0)
	playButton:SetTexture(addonID, GetPostingQueuePaused() and "Textures/Play.png" or "Textures/Pause.png")

	clearButton:SetPoint("BOTTOMLEFT", playButton, "BOTTOMRIGHT", 10, 0)
	clearButton:SetTexture(addonID, "Textures/Stop.png")
	
	queuePanel:SetPoint("BOTTOMLEFT", playButton, "TOPLEFT", 0, -5)
	queuePanel:SetPoint("BOTTOMRIGHT", clearButton, "TOPRIGHT", 0, -5)
	queuePanel:SetHeight(30)
	queuePanel:GetContent():SetBackgroundColor(0, 0, 0, 0.5)
	
	queueText:SetPoint("CENTER", queuePanel:GetContent(), "CENTER")
	local _, size = GetPostingQueueStatus()
	queueText:SetText(tostring(size))

	auctionFrame:SetWidth(400)
	auctionFrame:SetHeight(300)
	auctionFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", configuration.auctionFrameX or (UIParent:GetWidth() - auctionFrame:GetWidth()) / 2, configuration.auctionFrameY or (UIParent:GetHeight() - auctionFrame:GetHeight()) / 2)
	auctionFrame:SetVisible(false)
	auctionFrame:SetBackgroundColor(0, 0, 0, auctionOpacity)
	
	auctionBackground:SetAllPoints()
	auctionBackground:SetTexture(addonID, "Textures/AuctionFrame.png")
	auctionBackground:SetLayer(-9999)

	averageText:SetPoint("CENTERLEFT", auctionFrame, 0, 1/9, 10, 0)
	averageText:SetFontSize(16)
	averageText:SetFontColor(0.75, 0.75, 0.5, 1)
	averageText:SetText(L.lblAverage)
	
	lowestText:SetPoint("CENTERLEFT", auctionFrame, 0, 2/9, 10, 0)
	lowestText:SetFontSize(16)
	lowestText:SetFontColor(0.75, 0.75, 0.5, 1)
	lowestText:SetText(L.lblLowest)
	
	lastText:SetPoint("CENTERLEFT", auctionFrame, 0, 3/9, 10, 0)
	lastText:SetFontSize(16)
	lastText:SetFontColor(0.75, 0.75, 0.5, 1)
	lastText:SetText(L.lblLast)
	
	stackSizeText:SetPoint("CENTERLEFT", auctionFrame, 0, 4/9, 10, 0)
	stackSizeText:SetFontSize(16)
	stackSizeText:SetFontColor(0.75, 0.75, 0.5, 1)
	stackSizeText:SetText(L.lblStackSize)
	
	stackText:SetPoint("CENTERLEFT", auctionFrame, 0, 5/9, 10, 0)
	stackText:SetFontSize(16)
	stackText:SetFontColor(0.75, 0.75, 0.5, 1)
	stackText:SetText(L.lblStacks)
	
	priceText:SetPoint("CENTERLEFT", auctionFrame, 0, 6/9, 10, 0)
	priceText:SetFontSize(16)
	priceText:SetFontColor(0.75, 0.75, 0.5, 1)
	priceText:SetText(L.lblPrice)
	
	durationText:SetPoint("CENTERLEFT", auctionFrame, 0, 7/9, 10, 0)
	durationText:SetFontSize(16)
	durationText:SetFontColor(0.75, 0.75, 0.5, 1)
	durationText:SetText(L.lblDuration)

	averageMoney:SetPoint("CENTERLEFT", auctionFrame, 0.4, 1/9, 0, 2)
	averageMoney:SetPoint("CENTERRIGHT", auctionFrame, 1, 1/9, -10, 2)
	averageMoney:SetHeight(20)
	
	lowestMoney:SetPoint("CENTERLEFT", auctionFrame, 0.4, 2/9, 0, 2)
	lowestMoney:SetPoint("CENTERRIGHT", auctionFrame, 1, 2/9, -10, 2)
	lowestMoney:SetHeight(20)
	
	lastMoney:SetPoint("CENTERLEFT", auctionFrame, 0.4, 3/9, 0, 2)
	lastMoney:SetPoint("CENTERRIGHT", auctionFrame, 1, 3/9, -10, 2)
	lastMoney:SetHeight(20)

	stackSizeSlider:SetPoint("CENTERLEFT", auctionFrame, 0.4, 4/9, 0, 7)
	stackSizeSlider:SetPoint("CENTERRIGHT", auctionFrame, 1, 4/9, -10, 7)
	
	stackSlider:SetPoint("CENTERLEFT", auctionFrame, 0.4, 5/9, 0, 7)
	stackSlider:SetPoint("CENTERRIGHT", auctionFrame, 1, 5/9, -10, 7)
    
	priceSelector:SetPoint("CENTERLEFT", auctionFrame, 0.4, 6/9, 0, 0)
	priceSelector:SetPoint("CENTERRIGHT", auctionFrame, 1, 6/9, -10, 0)
	priceSelector:SetHeight(28)
    
	durationSlider:SetPoint("CENTERLEFT", auctionFrame, 0.4, 7/9, 10, 7)
	durationSlider:SetPoint("CENTERRIGHT", auctionFrame, 1, 7/9, -70, 7)
	durationSlider:SetRange(1, 3)
	
	hoursText:SetPoint("CENTERLEFT", auctionFrame, 1, 7/9, -50, 0)
	hoursText:SetPoint("CENTERRIGHT", auctionFrame, 1, 7/9, -10, 0)
	hoursText:SetText(SFormat(L.lblHours, 12))

	refreshButton:SetPoint("CENTER", auctionFrame, 0.25, 8/9)
	refreshButton:SetText(L.btnSearch)
	refreshButton:SetEnabled(false)
	
	postButton:SetPoint("CENTER", auctionFrame, 0.75, 8/9)
	postButton:SetText(L.btnPost)

	local itemTypeTable = {}
	local itemTypeOrder = {}
	local itemOffset = 1
	local unfolded = false
	local showHidden = false
	
	local function UpdateAuctionFrame(itemType)
		averageMoney:SetValue(0)
		lowestMoney:SetValue(0)
		lastMoney:SetValue(0)
		stackSizeSlider:SetRange(0, 0)
		stackSlider:SetRange(0, 0)
		priceSelector.itemType = nil
		priceSelector:SetValue(0)
		priceSelector:SetEnabled(false)
		durationSlider:SetPosition(defaultDuration)
		refreshButton.itemType = nil
		refreshButton:SetEnabled(false)
		postButton.itemType = nil
		postButton:SetEnabled(false)
		
		local function GetAuctionsCallback(itemType, auctions)
			local itemInfo = itemTypeTable[itemType]
			if not itemInfo then return end
		
			local lowPrice, accPrice, total = nil, 0, 0
			for auctionID, auctionData in pairs(auctions) do
				local buyPrice, stacks = auctionData.buyoutUnitPrice, auctionData.stack
				if buyPrice then
					lowPrice = lowPrice and lowPrice <= buyPrice and lowPrice or buyPrice
					accPrice = accPrice + buyPrice * stacks
					total = total + stacks
				end
			end
			lowPrice = lowPrice or 0
			local avgPrice = total > 0 and MFloor(accPrice / total) or 0
			
			local postConfig = APGadgetPostingConfigs[itemType] or {}
			postConfig.price = postConfig.price and postConfig.price > 0 and postConfig.price or nil
			
			averageMoney:SetValue(avgPrice)
			lowestMoney:SetValue(lowPrice)
			lastMoney:SetValue(postConfig.price or 0)
			local maxStack = MMax(MMin(itemInfo.stackMax or 1, itemInfo.stack), 1)
			stackSlider.stack = itemInfo.stack
			stackSizeSlider:SetRange(1, maxStack)
			stackSizeSlider:SetPosition(postConfig.stackSize or maxStack)
			priceSelector.itemType = itemType
			priceSelector:SetEnabled(true)
			priceSelector:SetValue(postConfig.price or (defaultPrice == "average" and avgPrice or lowPrice))
			if postConfig.duration then durationSlider:SetPosition(postConfig.duration) end
			refreshButton.itemType = itemType
			refreshButton:SetEnabled(IInteraction("auction"))
			postButton.itemType = itemType
			postButton:SetEnabled(priceSelector:GetValue() > 0)
		end
		
		if itemType then
			GetAuctions(function(auctions) GetAuctionsCallback(itemType, auctions) end, itemType)
		end		
	end
	
	local function RepopulateRibbon()
		for index, itemFrame in ipairs(itemFrames) do
			if #itemTypeOrder > 0 then
				local ind = index + itemOffset - ITEM_FRAME_EXTRA - 1
				while ind < 1 do ind = ind + #itemTypeOrder end
				while ind > #itemTypeOrder do ind = ind - #itemTypeOrder end

				local itemType = itemTypeOrder[ind]
				local itemInfo = itemType and itemTypeTable[itemType] or {}
				itemFrame:SetItem(itemType, itemInfo.icon, itemInfo.rarity)
				
				if index == ITEM_FRAME_EXTRA + 1 then
					itemName:SetText(itemInfo.name .. " (" .. itemInfo.stack .. ")")
					UpdateAuctionFrame(itemType)
				end
			else
				itemFrame:SetItem()
			end
		end
		if #itemTypeOrder <= 0 then
			itemName:SetText("")
			UpdateAuctionFrame(nil)
		end
	end
	
	local function ApplyOffset(amount)
		itemOffset = itemOffset + amount
		RepopulateRibbon()
	end
	
	local function RefreshItems()
		local slot = UISInventory()
		local items = IIList(slot)
		
		local lastItemType = nil
		if #itemTypeOrder > 0 then
			local ind = itemOffset
			while ind < 1 do ind = ind + #itemTypeOrder end
			while ind > #itemTypeOrder do ind = ind - #itemTypeOrder end
			lastItemType = itemTypeOrder[ind]
		end
		
		itemTypeTable = {}
		
		for _, itemID in pairs(items) do repeat
			if type(itemID) == "boolean" then break end 
			local ok, itemDetail = pcall(IIDetail, itemID)
			if not ok or not itemDetail or itemDetail.bound then break end
			
			local itemType = itemDetail.type
			if not showHidden and APGadgetHiddenItems[itemType] then break end
			
			itemTypeTable[itemType] = itemTypeTable[itemType] or { name = itemDetail.name, icon = itemDetail.icon, rarity = itemDetail.rarity, stack = 0, stackMax = itemDetail.stackMax, sell = itemDetail.sell }
			itemTypeTable[itemType].stack = itemTypeTable[itemType].stack + (itemDetail.stack or 1)
		until true end
		
		local pendingPosts = GetPostingQueue()
		for _, postingData in ipairs(pendingPosts) do
			if itemTypeTable[postingData.itemType] then
				itemTypeTable[postingData.itemType].stack = itemTypeTable[postingData.itemType].stack - postingData.amount
				if itemTypeTable[postingData.itemType].stack <= 0 then
					itemTypeTable[postingData.itemType] = nil
				end
			end
		end
		
		itemTypeOrder = {}
		for itemType in pairs(itemTypeTable) do TInsert(itemTypeOrder, itemType) end
		TSort(itemTypeOrder, function(a,b) if itemTypeTable[a].name == itemTypeTable[b].name then return a < b end return itemTypeTable[a].name < itemTypeTable[b].name end)
		
		if lastItemType then
			for index, itemType in ipairs(itemTypeOrder) do
				if itemType == lastItemType then
					itemOffset = index
					break
				end
			end
		end

		RepopulateRibbon()
	end
	
	local function UpdateInteraction(interaction, state)
		if interaction == "auction" then
			refreshButton:SetEnabled(state and refreshButton.itemType and true or false)
			scanButton.enabled = state
			scanButton.Event.MouseOut(scanButton)
		end
	end
	
	local function UpdatePostingQueue()
		playButton:SetTexture(addonID, GetPostingQueuePaused() and "Textures/Play.png" or "Textures/Pause.png")
		local _, size = GetPostingQueueStatus()
		queueText:SetText(tostring(size))
	end

	function itemRibbon.Event:WheelForward()
		ApplyOffset(-1)
	end
	
	function itemRibbon.Event:WheelBack()
		ApplyOffset(1)
	end
	
	for index, itemFrame in ipairs(itemFrames) do
		local frameOffset = index - ITEM_FRAME_EXTRA - 1
		function itemFrame.Event:LeftClick() ApplyOffset(frameOffset) end
		function itemFrame.Event:RightClick() 
			if self.lastItemType then 
				APGadgetHiddenItems[self.lastItemType] = not APGadgetHiddenItems[self.lastItemType] or nil 
				RefreshItems()
				ApplyOffset(0)
			end 
		end
	end
	
	function hideButton.Event:LeftClick()
		showHidden = not showHidden
		self:SetTexture(addonID, showHidden and "Textures/ShowIcon.png" or "Textures/HideIcon.png")
		RefreshItems()
	end
	
	function foldButton.Event:LeftClick()
		unfolded = not unfolded
		if unfolded then
			WT.FadeIn(auctionFrame, 0.5)
		else
			WT.FadeOut(auctionFrame, 0.5)
		end
	end
	
	function scanButton.Event:LeftClick()
		if not self.enabled then return end
		pcall(CAScan, { type="search", sort = "time", sortOrder = "descending" })
	end

	
	function scanButton.Event:MouseIn()
		if self.enabled then
			self:SetTexture(addonID, "Textures/RefreshMiniOn.png")
		else
			self:SetTexture(addonID, "Textures/RefreshMiniDisabled.png")
		end
	end
	
	function scanButton.Event:MouseOut()
		if self.enabled then
			self:SetTexture(addonID, "Textures/RefreshMiniOff.png")
		else
			self:SetTexture(addonID, "Textures/RefreshMiniDisabled.png")
		end
	end	
	
	function playButton.Event:LeftClick()
		SetPostingQueuePaused(not GetPostingQueuePaused())
	end
	
	function clearButton.Event:LeftClick()
		CancelAll()
	end
	
	local function MoneyClick(self)
		if priceSelector:GetEnabled() then
			priceSelector:SetValue(self:GetValue())
		end
	end
	averageMoney.Event.RightClick = MoneyClick
	lowestMoney.Event.RightClick = MoneyClick
	lastMoney.Event.RightClick = MoneyClick
	
	function stackSizeSlider.Event:PositionChanged(position)
		local stack = stackSlider.stack
		if stack and position and position > 0 then
			local maxNumber = MMax(MCeil(stack / position), 1)
			stackSlider:SetRange(1, maxNumber)
			stackSlider:SetPosition(maxNumber)
			if postButton.itemType then
				APGadgetPostingConfigs[postButton.itemType] = APGadgetPostingConfigs[postButton.itemType] or {}
				APGadgetPostingConfigs[postButton.itemType].stackSize = position
			end
		end
	end
	
	function priceSelector.Event:ValueChanged()
		local value = self:GetValue()
		postButton:SetEnabled(value > 0 and self.itemType and true or false)
		if self.itemType then
			APGadgetPostingConfigs[self.itemType] = APGadgetPostingConfigs[self.itemType] or {}
			APGadgetPostingConfigs[self.itemType].price = value
		end
	end
	
	function durationSlider.Event:SliderChange()
		local position = self:GetPosition()
		hoursText:SetText(SFormat(L.lblHours, (6 * 2 ^ position)))
		if postButton.itemType then
			APGadgetPostingConfigs[postButton.itemType] = APGadgetPostingConfigs[postButton.itemType] or {}
			APGadgetPostingConfigs[postButton.itemType].duration = position
		end
	end
	
	function refreshButton.Event:LeftPress()
		local itemType = self.itemType
		local ok, itemInfo = pcall(IIDetail, itemType)
		if not ok then itemInfo = itemType and itemTypeTable[itemType] or nil end
		if itemInfo then
			pcall(CAScan, { type = "search", index = 0, text = itemInfo.name, rarity = itemInfo.rarity or "common", category = itemInfo.category, sort = "pricePerUnit", sortOrder = "descending" })
		end
	end
	
	function postButton.Event:LeftPress()
		local itemType = self.itemType
		local stackSize = stackSizeSlider:GetPosition()
		local amount = stackSlider:GetPosition() * stackSize
		amount = MMin(amount, itemType and itemTypeTable[itemType] and itemTypeTable[itemType].stack or 0)
		local price = priceSelector:GetValue()
		local duration = durationSlider:GetPosition()
		
		if itemType and stackSize > 0 and amount > 0 and price > 0 then
			PostItem(itemType, stackSize, amount, price, price, 6 * 2 ^ duration)
		end
	end
	
	function auctionFrame.Event:LeftDown()
		if self.dragInfo then return end
		local mouse = Inspect.Mouse()
		self.dragInfo = 
		{
			x = mouse.x, 
			y = mouse.y,
			left = self:GetLeft(),
			top = self:GetTop(),
		}
	end
	function auctionFrame.Event:LeftUp()
		self.dragInfo = nil
	end
	function auctionFrame.Event:LeftUpoutside()
		self.dragInfo = nil
	end
	function auctionFrame.Event:MouseMove(x, y)
		if not self.dragInfo then return end
		local dx = x - self.dragInfo.x
		local dy = y - self.dragInfo.y
		self:ClearPoint("TOPLEFT")
		local newX, newY = self.dragInfo.left + dx, self.dragInfo.top + dy
		self:SetPoint("TOPLEFT", UIParent, "TOPLEFT", newX, newY)
		wtxGadgets[configuration.id].auctionFrameX = newX
		wtxGadgets[configuration.id].auctionFrameY = newY
	end		

	local oldSetVisible = itemRibbon.SetVisible
	itemRibbon.SetVisible = function(self, visible)
		if not visible then auctionFrame:SetVisible(false) end
		oldSetVisible(self, visible)
	end
	
	local startTime = ITFrame() + LOAD_DELAY
	TInsert(Event.Item.Slot, { RefreshItems, addonID, addonID .. ".Ribbon.OnItemSlot" })
	TInsert(Event.Item.Update, { RefreshItems, addonID, addonID .. ".Ribbon.OnItemUpdate" })
	TInsert(Event.Interaction, { UpdateInteraction, addonID, addonID .. ".AuctionFrame.OnInteraction" })
	TInsert(Event.System.Update.Begin, { function() if startTime and ITFrame() > startTime then RefreshItems() startTime = nil end end, addonID, addonID .. ".Ribbon.OnStart" })
	TInsert(Event.LibPGC.AuctionData, { RepopulateRibbon, addonID, addonID .. ".AuctionFrame.OnAuctionData" })
	TInsert(Event.LibPGC.PostingQueueChanged, { RefreshItems, addonID, addonID .. ".Ribbon.OnPostingQueueChanged" })
	TInsert(Event.LibPGC.PostingQueueStatusChanged, { UpdatePostingQueue, addonID, addonID .. ".Ribbon.OnPostingQueueStatusChanged" })
	RefreshItems()
	
	return itemRibbon
end

local function ConfigDialog(container)	
    dialog = WT.Dialog(container)
        :Label(L.cnfIntro)
		:Select("extraItems", L.cnfItemsOption, "7",
		{
			{text = L.cnfItems7, value = "7"},
			{text = L.cnfItems9, value = "9"},
			{text = L.cnfItems11, value = "11"},
		}, false)
		:Select("auctionOpacity", L.cnfOpacityOption, "50",
		{
			{text = "50 %", value = "50"},
			{text = "60 %", value = "60"},
			{text = "70 %", value = "70"},
			{text = "80 %", value = "80"},
			{text = "90 %", value = "90"},
			{text = "100 %", value = "100"},
		}, false)
		:Select("defaultPrice", L.cnfPriceOption, "average",
		{
			{text = L.cnfPriceAverage, value = "average"},
			{text = L.cnfPriceLowest, value = "lowest"},
		}, true)
		:Select("defaultDuration", L.cnfDurationOption, "12",
		{
			{text = L.cnfDuration12, value = "12"},
			{text = L.cnfDuration24, value = "24"},
			{text = L.cnfDuration48, value = "48"},
		}, false)
end

local function GetConfiguration()
    return dialog:GetValues()
end

local function SetConfiguration(config)
    dialog:SetValues(config)
end

TInsert(Event.Addon.SavedVariables.Load.End, { function(addonId) if addonId == addonID then InitializeSavedVars() end end, addonID, addonID .. ".OnSavedVarsLoad" })

InitializeSavedVars()

WT.Gadget.RegisterFactory(addonID,
{
    name = "Auction Post Gadget",
    description = L.gdgDescription,
    author = "Baanano",
    version ="0.4.1",
	iconTexAddon = addonID,
	iconTexFile = "Textures/GadgetIcon.png",
    ["Create"] = CreateGadget,
	["ConfigDialog"] = ConfigDialog,
	["GetConfiguration"] = GetConfiguration, 
	["SetConfiguration"] = SetConfiguration, 
})
